#!/usr/bin/python
# -*- coding: utf-8 -*-
#######################################################################
# File  : artist_tool.py
# Author: Souheil CHELFOUH
#         mailto:souheil@chelfouh.com
#         http://www.trollskog.com
# Date  : 2006-02-11
#######################################################################

__author__	= 'Souheil CHELFOUH <souheil@chelfouh.com>'
__docformat__	= 'plaintext'

import urllib, string, struct, re

from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo 
from Products.CMFCore.utils import UniqueObject, getToolByName

from Products.TPArtist.config import PROJECTNAME
from artist_tool_config import VIEW_PERMISSION, UNIQUE_ID

class TPArtistTool(UniqueObject, SimpleItem):
    """
    """
    
    id          = UNIQUE_ID
    title       = "TPArtist Tool"
    meta_type   = "TPArtistTool"

    associate_type = {"medias" : "Album",
                      "styles" : "Artist",}
    
    security = ClassSecurityInfo()

    security.declareProtected(VIEW_PERMISSION, 'handleConfigletEdit')
    def handleConfigletEdit(self, args = None, fieldset = None):
        """
        """
        if fieldset and args:
            keys  = args[fieldset].split('\n')
            keys  = [key.strip() for key in keys]
            force = False
            if hasattr(args, 'force_' + fieldset):
                force = args['force_' + fieldset]
            return self.setType(keys, force, self.associate_type[fieldset])
            return "%s policies handled" % self.associate_type[fieldset]

    
    security.declareProtected(VIEW_PERMISSION, 'setType')
    def setType(self, keys, force, metatype):
        """
        Set the Subject metadata with the values in args (REQUEST)
        """       
        pmd  = getToolByName(self, "portal_metadata")	

        try:
            pmd.removeElementPolicy('Subject', metatype)
        except:
            pass
        
        pmd.addElementPolicy(element = 'Subject',
                             content_type = metatype,
                             is_required = 1,
                             supply_default = "",
                             default_value = "",
                             enforce_vocabulary = force,
                             allowed_vocabulary = keys,
                             REQUEST = None)
        
        return "%s policies updated !" % metatype


    security.declareProtected(VIEW_PERMISSION, 'getVocabularyEnforcing')
    def getVocabularyEnforcing(self, type):
        """
        Sends back the Subject vocabulary enforcement policy
        """
        pmd  = getToolByName(self, "portal_metadata")
        return pmd.listPolicies(typ = self.associate_type[type])[5][1].enforceVocabulary()


    security.declareProtected(VIEW_PERMISSION, 'getValues')
    def getValues(self, type):
        """
        Sends back the Subject vocabulary
        """
        pmd  = getToolByName(self, "portal_metadata")
        keys = pmd.listAllowedSubjects(content_type = self.associate_type[type])
        return keys


    #################
    ### CDDB CONF ###
    #################
    
    default_server = 'http://freedb.freedb.org/~cddb/cddb.cgi'

    ####################
    ### CDDB METHODS ###
    ####################

    def getDecentTitle(self, title):
        """
        """
        if title.count('/'):
            title = title.split('/')[1].strip()
            if not len(title):
                title = "untitled"
        title = title.lower().capitalize()
        return title


    def getMeNiceId(self, title):
        """
        """
        plone_tool = getToolByName(self, 'plone_utils', None)
        new_id     = plone_tool.normalizeString(title)
        return new_id


    def getMeValidId(self, container, title):
        """
        """
        ids        = container.objectIds()
        new_id     = self.getMeNiceId(title)
        invalid_id = new_id in ids

        if not invalid_id:
            return new_id

        new_id     = new_id + str(int(self.ZopeTime()))
        invalid_id = new_id in ids

        if not invalid_id:
            return new_id
        
        return None

    
    def albumRead(self, category = None, disc_id = None, server_url=default_server, 
                  user="plone", host="plone.org", client_name="plone2.1.1",
                  client_version=6):

        if not category or not disc_id:
            return [ "500", None ]

        url = "%s?cmd=cddb+read+%s+%s&hello=%s+%s+%s+%s&proto=%i" % \
              (server_url, category, disc_id, user, host, client_name,
               client_version, client_version)

        response = urllib.urlopen(url)
    
        header = string.split(string.rstrip(response.readline()), ' ', 3)
        
        header[0] = string.atoi(header[0])
        if header[0] == 210 or header[0] == 417: # success or access denied
            reply = []

            for line in response.readlines():
                line = string.rstrip(line)
                if line == '.':
                    break;

                line = string.replace(line, r'\t', "\t")
                line = string.replace(line, r'\n', "\n")
                line = string.replace(line, r'\\', "\\")

                reply.append(line)

            if header[0] == 210:	# success, parse the reply
                return [ header[0], self.parse_read_reply(reply) ]
            else:			# access denied. :(
                return [ header[0], reply ]
        else:
            return [ header[0], None ]


    def parse_read_reply(self, comments):
        
        len_re = re.compile(r'#\s*Disc length:\s*(\d+)\s*seconds')
        revis_re = re.compile(r'#\s*Revision:\s*(\d+)')
        submit_re = re.compile(r'#\s*Submitted via:\s*(.+)')
        keyword_re = re.compile(r'([^=]+)=(.*)')
        
        result = {}
        
        for line in comments:
            keyword_match = keyword_re.match(line)
            if keyword_match:
                (keyword, data) = keyword_match.groups()
                
                if result.has_key(keyword):
                    result[keyword] = result[keyword] + data
                else:
                    result[keyword] = data
                    continue

                len_match = len_re.match(line)
                if len_match:
                    result['disc_len'] = int(len_match.group(1))
                    continue
                
                revis_match = revis_re.match(line)
                if revis_match:
                    result['revision'] = int(revis_match.group(1))
                    continue

                submit_match = submit_re.match(line)
                if submit_match:
                    result['submitted_via'] = submit_match.group(1)
                    continue
                
        return result


    def getSearchPage(self, artist = None):
        """
        Fetch the disc_id and style info from
        the freeDB portal, with an horrible GET
        """
        if artist:
            artist = artist.replace(' ', '+')
            host = "www.freedb.org"
            page = "freedb_search.php"
            args = "words=%s&allfields=NO&fields=artist&allcats=YES&grouping=none" % artist
            url = "http://%s/%s?%s" % (host, page, args)
        
            resp = urllib.urlopen(url)
            data = resp.read()
            m = re.findall("(<a href=\".*\?cat=(.*)&id=(.*)\">([^<]+)<\/a>)", data)
            return m
        return None


    def getAlbumsFromArtist(self, artist = None):
        """
        """
        if artist:
            albums = self.getSearchPage(artist)
            return albums
        return None


InitializeClass(TPArtistTool)
